/**********************************************************************
 *                                                                    *
 * CLOSE.C, version 1.0                                               *
 *                                                                    *
 * This program sends a WM_CLOSE message to the window(s) whose title *
 * match(es) the argument given on the command line. A '*' at the END *
 * of the argument is interpreted in the common sense.                *
 * This gracefully closes the program in question, giving it time to  *
 * clean up. To kill (e.g. non-responding) applications, see the KILL *
 * utility which comes with the Resource Kit and as source with VC++. *
 *                                                                    *
 * Copyright (C) 1997 by Alexander Frink                              *
 *                       Hermann Schauss Str. 8                       *
 *                       65232 Taunusstein                            *
 *                       Germany                                      *
 *                       Alexander.Frink@Uni-Mainz.DE                 *
 *                                                                    *
 * This program is free software; you can redistribute it and/or      *
 * modify it under the terms of the GNU General Public License        * 
 * as published by the Free Software Foundation; either version 2     *
 * of the License, or (at your option) any later version.             *
 *                                                                    *
 * This program is distributed in the hope that it will be useful,    *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the       *
 * GNU General Public License (file name: LICENSE) for more details.  *
 *                                                                    *
 * You should have received a copy of the GNU General Public License  * 
 * along with this program; if not, write to the Free Software        *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
 *                                                                    *
 **********************************************************************/

#include <windows.h>
#include <stdio.h>

#define MAXLEN 1024

UCHAR szTitleToClose[MAXLEN];
BOOL bWildcard=FALSE;
DWORD dwClosed=0;

BOOL CALLBACK ListWindows(HWND hWnd, LPARAM lparam)
{
    UCHAR szTitle[MAXLEN];
    GetWindowText(hWnd,szTitle,MAXLEN);
	if (bWildcard) {
	    if (strncmp(szTitle,szTitleToClose,strlen(szTitleToClose))==0) {
			PostMessage(hWnd,WM_CLOSE,0,0);
			dwClosed++;
		}
	} else {
		if (strcmp(szTitle,szTitleToClose)==0) {
			PostMessage(hWnd,WM_CLOSE,0,0);
			dwClosed++;
		}
    }
    return TRUE;
}

int main(int argc, char *argv[])
{
	if (argc!=2) {
		printf("Usage: %s title\n",argv[0]);
		printf("       Closes the window(s) with the title 'title' (in the title bar).\n");
		printf("       'title' may end with a '*' as a wildcard.\n");
		printf("       'title' is case-sensitive.\n");
		printf("       Enclose 'title' with quotes (\"\") if it contains blanks.\n\n");
		printf("This is free software under the GNU General Public License.\n");
		printf("Copyright (C) 1997 by Alexander Frink\n");
		return 1;
	}
	strcpy(szTitleToClose,argv[1]);
 	if (szTitleToClose[strlen(szTitleToClose)-1]=='*') {
		bWildcard=TRUE;
		szTitleToClose[strlen(szTitleToClose)-1]='\0';
	}
	
	EnumWindows(ListWindows,0);

	if (dwClosed==0) {
		printf("No matching windows found!\n");
		return 1;
	}
	printf("OK, successfully closed %i window(s) with title '%s'.\n",dwClosed,argv[1]);
    return 0;
}
